#ifndef Analogger_h
#define Analogger_h

// **********************************************
// **        NOTES
// **********************************************

/*
 * Analogger is a simple datalogging script that uses the teensy as a single channel adc.  The goal is to have a simple interface that takes a few configuration parameters and 
 * does the work of setting up the adc and communication with a host computer.  The adc setup itself takes one parameter, frequency.  Based on this,
 * the adc is setup loosely based on the speed tests done by Pedvide in the continuous read sample script that ships with teensyduino.
 * For the datalogging, the other parameters are whether or not a hardware trigger, or a software trigger is used to start the logging, how many times you
 * want the logger to trigger, and how much time is spent recording after each trigger.
*/



#include "Arduino.h"
#include <ADC.h>
#include <RingBuffer.h>
// #define RING_BUFFER_DEFAULT_BUFFER_SIZE 4096
//#define NUM_USB_BUFFERS  16
//#define TX_PACKET_LIMIT 32 

#define ANA_TYPE_PUSH 240
#define ANA_TYPE_REQ  170
#define ANA_TYPE_ECHO 85
#define ANA_TYPE_DATA 54

#define ANA_SUB_FREQ  255
#define ANA_SUB_SEC   240
#define ANA_SUB_NUM_TRIG  219
#define ANA_SUB_DELAY  194
#define ANA_SUB_HWPIN  170
#define ANA_SUB_ADCPIN  153
#define ANA_SUB_HWENAB  102
#define ANA_SUB_SWENAB  85
#define ANA_SUB_CONFIG_ENAB  54
#define ANA_SUB_ADCSTATUS  45

#define ADC_START 1111
#define ADC_END 9999
#define SW_TRIGGER 5555
#define MARCO 12345
#define POLO 54321

class Analogger {

    public:
        
          uint32_t sw_trigger_enable;  //Does the program activate logging on a software trigger?
          
          uint32_t pdb_trigger_frequency;     //Frequnecy to sample at
          
          uint32_t seconds;     //Time to sample for per trigger event, I did this instead of defining samples, seems like it's more useful this way. If you don't like it, swap this for the calculations of numSamples in the setup.
          
          uint32_t time_delay;     //Should there be a time delay after trigger before recording? measured in millis
          
          uint32_t num_of_triggers;    //Number of times the logger can be triggered.  Note: If both hardware and software triggering are enabled, this may not behave as you expect.
          
          
          
          //**************** FUNCTIONS  *******************

          //Constructor, initialize variables
          Analogger();
          
          //User Config Function
          void Config(bool SWEnabled = false, uint32_t Freq = 10000, uint32_t recordTime = 1, uint32_t Triggers = 1, uint32_t Delay = 0);

          // When a message from the host computer is received, this parses out the pieces so they can be used
          void parseMessage(uint8_t tdxArray[64], uint8_t &Type, uint8_t &Sub, uint32_t &Value);

          // Use when expecting a message, holds the program for some amount of time to receive a message
          bool waitForMessage(uint32_t waitTime, uint8_t reply[64]);

          // Send a previously formatted message and wait for a reply. This one is for config requests, config pushes, and response echos.
          uint32_t sendMessage(uint8_t Type, uint8_t SubType, uint32_t Value);

          // Send a previously formatted message and wait for a reply. This one is for data messsages
          void sendMessage(uint8_t Type, uint32_t SerNum);

          //Function to use for events triggered by hardware events
          void hardwareTrigger();

          //Function to use for events triggered by communication from host computer
          void softwareTrigger();
          
          //Function to be placed in the ADC Interrupt Service Routine
          void isrFunc();
          

    private:

          //Constant Pin Values
          static const uint32_t adc_pin = A9;        //Which pin to read the analog samples from.  right now, only single ended readings of ADC0 are allowed.
          static const uint32_t hw_trigger_pin = 4;  //The pin to handle hardware triggers of the datalogging.  digitalReadFast requires a constant, so this cannot be configurable at runtime.
          static const uint8_t LEDpin = 13;   //LED pin to display communication setup.
          
          //Don't change this.  I added it for readability of code, not to change.  64 Bytes is the optimal usb transfer size
          static const uint8_t tx_buff_size = 60;
          static const uint8_t message_size = 64;
          
          //Configuration Parameters

          uint32_t hw_trigger_enable;   //Does the program activate logging on a hardware trigger?  Note: currently only accepts digitalHigh as a trigger.  Cannot trigger on low.
          uint32_t AnaloggerStatus;
          uint8_t sample_resolution;  //Resoultion of each analog reading.  If using default values, choices are 8 or 12.  The number here don't matter, will be changed during setup regardless.
          uint8_t num_avgs;            //Number of averages of each analog reading.  If slow, this may be as high as 32, if at all fast, 1.  The number here don't matter, will be changed during setup regardless.
          uint32_t time_trigger_start; //time trigger is activated in millis.  Used with time_delay.
          uint32_t numSamples; //A placeholder for the number of samples to be recorded.  Computed in setup.
                              //TODO
                              uint32_t config_enabled;     //Does the program enter configuration phase at beginning of loop?  NOT CURRENTLY ENABLED
          
          //Create the ADC class
          ADC adc;
          
          //Buffers for transmitting
          uint8_t message[message_size];
          uint8_t reply[message_size];
          uint8_t zeros[message_size];
          
          bool isConnected;
          
          //Buffer for ADC
          RingBuffer *buffer = new RingBuffer;
          volatile size_t write_pos;
          volatile size_t read_pos;
          volatile uint16_t adc_val;
          uint8_t tx_buffer_index;

          //Message Variables
          uint8_t replyType;
          uint8_t replySub;
          uint32_t replyValue;
          
          //These are flags to tell the teensy when to stop recording
          bool start_read;
          bool overrun;
          bool notTriggered;
          
          //These are for buffer overflow conditions.
          uint32_t readCounter;
          uint32_t error_count;
          bool randomCounter;
          
          uint32_t Serial_Number;


          //******************** FUNCTIONS *********************

          //Set up the ADC based on sample frequency.  This is what keeps Analogger usage simple.  Set a frequency and run.
          void adc_setup(uint32_t sampleFrequency);

          //Create the connection with a host computer.
          void makeConnection();
          
          //Clear the serial line of garbage.
          void clearRxBuffer(uint32_t clearTime);

          // Take reading from the ADC buffer and move it into a transmit message buffer.
          void ADCRecord(RingBuffer *buffer, uint8_t TXbuffer[64], uint8_t &TXindex, uint8_t resolution);

          // Take in some information and format a message to send to a host computer.  This one is for config requests, config pushes, and response echos.
          void createMessage(uint8_t Type, uint8_t Sub, uint32_t Value);

          // Take in some information and format a message to send to a host computer.  This one is for data messsages
          void createMessage(uint8_t Type, uint32_t SerialNum);

          // Tell the host computer the message was received.
          void replyToMessage(uint8_t message[64]);

          // Fire off an analogger trigger, really the meat of the entire code.
          void analoggerTrigger();


};


#endif
